from datetime import timedelta

from flask import Flask
from flask_sqlalchemy import SQLAlchemy

from management.data import shared
from portal.database import Worker
from portal.tools import time


def create_project(factory, test_course, name: str):
    factory.create_project(
        course=test_course, name=name.upper(), codename=name, config=dict(
            file_whitelist="*.*",
            test_files_source='git@gitlab.fi.muni.cz:xstanko2/pb071-mini-template.git',
            test_files_subdir=name,
            pre_submit_script="",
            post_submit_script="",
            submission_scheduler_config="",
            submission_parameters="{\"type\":\"text\"}",
            submissions_allowed_from=time.current_time() - timedelta(days=1),
            submissions_allowed_to=time.current_time() + timedelta(days=300),
            archive_from=time.current_time() + timedelta(days=330)
        )
    )


def init_c_data(app: Flask, db: SQLAlchemy):
    with app.app_context():
        factory = shared.DataFactory(db)
        # course
        cpp = factory.create_course(codename='PB071', name='Programování v jazyce C')

        # groups
        seminar_kontr = factory.create_group(course=cpp, name="stundets")
        teachers_group = factory.create_group(course=cpp, name="teachers")

        projects = []
        for num in range(2, 13):
            name = f"mini0{num}"
            projects.append(create_project(factory, cpp, name))

        # roles
        students = factory.scaffold_role(role_type='student', course=cpp)
        teachers = factory.scaffold_role(role_type='teacher', course=cpp)
        owner = factory.scaffold_role(role_type='owner', course=cpp)
        workers = factory.scaffold_role(role_type='worker', course=cpp)

        # Projects in groups
        db.session.add_all([students, teachers, owner, teachers_group, seminar_kontr])

        # workers
        worker = Worker.query.filter_by(codename='executor').first()
        worker.roles.append(workers)
        db.session.add_all([worker])
        # Commit to the DB
        db.session.commit()
