import logging
from typing import List

from gitlab.v4 import objects

from portal.facade.general_facade import GeneralFacade
from portal.service.gitlab import GitlabService

log = logging.getLogger(__name__)


class GitlabFacade(GeneralFacade):
    def __init__(self):
        super().__init__(GitlabService)

    @property
    def user_gl_token(self) -> str:
        return self._request.cookies.get('gitlab_token')

    @property
    def gl_service(self) -> GitlabService:
        return self._service

    def list_projects(self, **kwargs) -> List[objects.Project]:
        gitlab_token = self.user_gl_token
        log.debug(f"[GL_PROJ] List projects in gitlab: {kwargs}")
        return self.gl_service.list_projects(token=gitlab_token, **kwargs)

    def list_members(self, project_name, **kwargs):
        gitlab_token = self.user_gl_token
        log.debug(f"[GL_PROJ] List members for project({project_name}): {kwargs}")
        return self.gl_service.list_members(token=gitlab_token,
                                            project_name=project_name, **kwargs)

    def get_project(self, project_name, **kwargs):
        gitlab_token = self.user_gl_token
        log.debug(f"[GL_PROJ] Get project({project_name}): {kwargs}")
        return self.gl_service.get_project(token=gitlab_token,
                                           project_name=project_name, **kwargs)

    def add_member_to_project(self, **kwargs):
        gitlab_token = self.user_gl_token
        log.info(f"[GL_PROJ] Add member project in gitlab: {kwargs}")
        return self.gl_service.add_member_to_project(token=gitlab_token, **kwargs)
