import flask
from flask_restplus import Resource

from portal import facade
from portal.database.models import Client
from portal.service.auth import AuthService
from portal.service.find import FindService
from portal.service.permissions import PermissionsService
from portal.service.services_collection import ServicesCollection


class FacadesHolder:
    def __init__(self):
        self._reviews = facade.ReviewsFacade()
        self._users = facade.UsersFacade()
        self._courses = facade.CoursesFacade()
        self._roles = facade.RolesFacade()
        self._groups = facade.GroupsFacade()
        self._secrets = facade.SecretsFacade()
        self._workers = facade.WorkersFacade()
        self._submissions = facade.SubmissionsFacade()
        self._projects = facade.ProjectsFacade()
        self._gitlab = facade.GitlabFacade()

    @property
    def users(self) -> facade.UsersFacade:
        return self._users

    @property
    def courses(self) -> facade.CoursesFacade:
        return self._courses

    @property
    def groups(self) -> facade.GroupsFacade:
        return self._groups

    @property
    def roles(self) -> facade.RolesFacade:
        return self._roles

    @property
    def secrets(self) -> facade.SecretsFacade:
        return self._secrets

    @property
    def submissions(self) -> facade.SubmissionsFacade:
        return self._submissions

    @property
    def projects(self) -> facade.ProjectsFacade:
        return self._projects

    @property
    def workers(self) -> facade.WorkersFacade:
        return self._workers

    @property
    def reviews(self) -> facade.ReviewsFacade:
        return self._reviews

    @property
    def gitlab(self) -> facade.GitlabFacade:
        return self._gitlab


class CustomResource(Resource):
    @property
    def rest(self) -> ServicesCollection:
        if not hasattr(self, '__rest_service'):
            setattr(self, '__rest_service', ServicesCollection())
        return getattr(self, '__rest_service')

    @property
    def facades(self) -> FacadesHolder:
        if not hasattr(self, '__facades'):
            setattr(self, '__facades', FacadesHolder())
        return getattr(self, '__facades')

    @property
    def find(self) -> FindService:
        if not hasattr(self, '__find_service'):
            setattr(self, '__find_service', FindService())
        return getattr(self, '__find_service')

    @property
    def permissions(self) -> PermissionsService:
        if not hasattr(self, '__perm_service'):
            setattr(self, '__perm_service', PermissionsService())
        return getattr(self, '__perm_service')

    @property
    def auth(self) -> AuthService:
        if not hasattr(self, '__auth_service'):
            setattr(self, '__auth_service', AuthService())
        return getattr(self, '__auth_service')

    @property
    def client(self) -> Client:
        return self.auth.client

    @property
    def client_name(self):
        if self.client:
            return self.client.log_name
        return 'Unknown client'

    @property
    def request(self) -> flask.Request:
        return flask.request

    @property
    def flask_app(self) -> flask.Flask:
        return flask.current_app
