import gitlab
from gitlab.v4 import objects

from portal.service.general import GeneralService


class GitlabService(GeneralService):
    @property
    def gitlab_factory(self):
        from portal import gitlab_factory
        return gitlab_factory

    @property
    def kontr_gitlab(self) -> gitlab.Gitlab:
        private_token = self._config.get('GITLAB_PRIVATE_TOKEN')
        if not private_token:
            return None
        return self.client(private_token=private_token)

    @property
    def kontr_gl_user(self) -> objects.User:
        return self.kontr_gitlab.user

    @property
    def kontr_username(self) -> str:
        return self._config.get('GITLAB_KONTR_USERNAME') or self.kontr_gl_user.username

    @property
    def user(self):
        return self._entity

    def client(self, **kwargs) -> gitlab.Gitlab:
        instance = self.gitlab_factory.instance(**kwargs)
        instance.auth()
        return instance

    def current_user(self, token: str = None, **kwargs):
        if not token:
            return None
        client = self.client(oauth_token=token, **kwargs)
        return client.user

    def list_projects(self, token: str, **kwargs):
        """
        Args:
            token(str): Gitlab access token

        Returns:
        DOC: https://python-gitlab.readthedocs.io/en/stable/gl_objects/projects.html
        """
        client = self.client(oauth_token=token)
        projects: objects.ProjectManager = client.projects
        return projects.list(all=True, **kwargs)

    def list_members(self, token: str, project_name: str, **kwargs):
        members = self._project_members(token=token, project_name=project_name)
        return members.list(**kwargs)

    def _project_members(self, token: str, project_name: str) -> objects.ProjectMemberManager:
        project = self.get_project(project_name=project_name, token=token)
        if not project:
            return None
        return project.members

    def get_project(self, token, project_name, **kwargs):
        client = self.client(oauth_token=token)
        projects: objects.ProjectManager = client.projects
        full_proj_name = client.user.username + '/' + project_name
        project: objects.Project = projects.get(full_proj_name, **kwargs)
        return project

    def add_member_to_project(self, token: str, project_name: str,
                              member_id=None, access_level=None, **kwargs):
        members = self._project_members(token=token, project_name=project_name)
        member_id = member_id or self.kontr_gl_user.id
        access_level = access_level or gitlab.REPORTER_ACCESS
        params = {'user_id': member_id.id, 'access_level': access_level}
        return members.create(params, **kwargs)

